/*
 * Project gbjFilterExponential
 * Description: Exponential filter for analog input data
 * Library Author: Libor Gabaj
 * This example author:  Mark W Kiehl  /  Mechatronic Solutions LLC
 * GitHub: https://github.com/mrkaleArduinoLib/gbj_filter_exponential
 * GitHub username: mrkale
 * License:   MIT License  
 *            https://github.com/mrkaleArduinoLib/gbj_filter_exponential/blob/master/LICENSE.txt
 */

const byte pinBuiltInLED = D7;
//////////////////////////////////////////////////////////////////////////////
// (Particle publish inteval limited to 1 event/sec)
// Sample every 1 ms (timerSampleInterval), filtering each sample with exponential 
// filter, and compiling a running sum and the number of samples.
// Every timerPublishInterval calculate the average for a analog input and then 
// publish the value to the Particle Cloud. 
const byte timerSampleInterval = 1;  // 1 ms = 1000 us = 0.001 sec = 1 kHz
unsigned long timerSampleLast = 0;  // timer
const unsigned long timerPublishInterval = 12000;  
unsigned long timerPublishLast = 0;  // timer
unsigned long publishCount = 0;
const byte ADCbits = 12;  // Argon, Boron, Xenon have 12-bit ADC
unsigned long samples = 0;
byte nextPublishedItem = 0;

unsigned int A0Fil = 0;
double A0mV = 0.0;

unsigned int A1Fil = 0;
double A1mV = 0.0;

unsigned int A2Fil = 0;
double A2mV = 0.0;

unsigned int A3Fil = 0;
double A3mV = 0.0;

unsigned int A4Fil = 0;
double A4mV = 0.0;

//////////////////////////////////////////////////////////////////////////////
// Exponential Filter
// https://github.com/mrkaleArduinoLib/gbj_filter_exponential
// Includes dependencies for the Particle and Arduino platform.
#include "gbj_filter_exponential.h"
// Smooth Factor parameter below is within the range of 0.0 to 1.0
// 0.623  the sampling time interval is equal to the time constant
// 0.5    regular running average
// 0.2    the sampling time interval ~4.5 times shorter than the time constant.
// 0.1    the sampling time interval ~9.5 times shorter than the time constant.
// 0.01   the sampling time interval ~100 times shorter than the time constant.
gbj_filter_exponential FilterA0 = gbj_filter_exponential(0.5);
gbj_filter_exponential FilterA1 = gbj_filter_exponential(0.5);
gbj_filter_exponential FilterA2 = gbj_filter_exponential(0.5);
gbj_filter_exponential FilterA3 = gbj_filter_exponential(0.5);
gbj_filter_exponential FilterA4 = gbj_filter_exponential(0.5);
// Initialize objects from the lib
//Gbj_filter_exponential gbj_filter_exponential;
/////////////////////////////////////////////////////////////////////////

void setup() {
  Mesh.off(); // Turn the Mesh Radio off
  pinMode(pinBuiltInLED, OUTPUT);
  digitalWrite(pinBuiltInLED, LOW);
  pinMode(A0, INPUT);
  pinMode(A1, INPUT);
  pinMode(A2, INPUT);
  pinMode(A3, INPUT);
  pinMode(A4, INPUT);

  // Call functions on initialized library objects that require hardware
  //gbj_filter_exponential.begin();


  timerPublishLast = millis();
  timerSampleLast = millis();
} // setup()

void loop() {

  // Use the library's initialized objects and functions
  //gbj_filter_exponential.process();

  if (timerSampleLast > millis())  timerSampleLast = millis();
  if ((millis() - timerSampleLast) > timerSampleInterval) {
    ReadAnalogInputs();
    timerSampleLast = millis();
  } // timer
  
  if (timerPublishLast > millis())  timerPublishLast = millis();
  if ((millis() - timerPublishLast) > timerPublishInterval) {
    CalcAvgAndPublishAnalogInputs();
    timerPublishLast = millis();
  } // timer

} // loop()


void ReadAnalogInputs() {
    A0Fil += FilterA0.getValue(analogRead(A0));
    A1Fil += FilterA1.getValue(analogRead(A1));
    A2Fil += FilterA2.getValue(analogRead(A2));
    A3Fil += FilterA3.getValue(analogRead(A3));
    A4Fil += FilterA4.getValue(analogRead(A4));
    samples++;
} // ReadAnalogInputs()


void CalcAvgAndPublishAnalogInputs() {
    // Calculate the average ADC values and update global double
    // variables A0mV .. A5mV and publish on variable per execution
    // of this function in sequence to the Particle Cloud.
    // The built-in blue LED on D7 will turn on if any publish error occurs.
  
    publishCount++;
    // Argon, Boron, Xenon have 12-bit ADC.  2^12 = 4096
    A0mV = ((A0Fil / samples) * (3300.0 / (double(4096))));
    A0Fil = 0;

    A1mV = ((A1Fil / samples) * (3300.0 / (double(4096))));
    A1Fil = 0;

    A2mV = ((A2Fil / samples) * (3300.0 / (double(4096))));
    A2Fil = 0;

    A3mV = ((A3Fil / samples) * (3300.0 / (double(4096))));
    A3Fil = 0;

    A4mV = ((A4Fil / samples) * (3300.0 / (double(4096))));
    A4Fil = 0;

    byte iResult = 0;
    // Ignore the first value calculated from the first 2461 samples
    if (publishCount > 1) {
      // Publish to the Particle Cloud.
      if (nextPublishedItem == 0) {
        iResult = Particle.publish("A0mV", String::format("%.2f",A0mV), PRIVATE);
        if (iResult != 1) digitalWrite(pinBuiltInLED, HIGH);
        nextPublishedItem = 1;
      } else if (nextPublishedItem == 1) {
        iResult = Particle.publish("A1mV", String::format("%.2f",A1mV), PRIVATE);
        if (iResult != 1) digitalWrite(pinBuiltInLED, HIGH);
        nextPublishedItem = 2;
      } else if (nextPublishedItem == 2) {
        iResult = Particle.publish("A2mV", String::format("%.2f",A2mV), PRIVATE);
        if (iResult != 1) digitalWrite(pinBuiltInLED, HIGH);
        nextPublishedItem = 3;
      } else if (nextPublishedItem == 3) {
        iResult = Particle.publish("A3mV", String::format("%.2f",A3mV), PRIVATE);
        if (iResult != 1) digitalWrite(pinBuiltInLED, HIGH);
        nextPublishedItem = 4;
      } else if (nextPublishedItem == 4) {
        iResult = Particle.publish("A4mV", String::format("%.2f",A4mV), PRIVATE);
        if (iResult != 1) digitalWrite(pinBuiltInLED, HIGH);
        nextPublishedItem = 0;
      }
    }
    samples = 0;  
} // CalcAvgAndPublishAnalogInputs()
